@@ -111,7 +111,6 @@ sub setId {
 #
 sub _realize {
     my $self     = shift;
-    my $script   = IWL::Script->new;
     my $id       = $self->getId;
     my $selected = 0;
 
@@ -119,9 +118,8 @@ sub _realize {
     foreach my $tab (@{$self->{__tabs}}) {
         last if $selected = $tab->isSelected;
     }
-    $self->{__tabs}[0]->setSelected(1) if !$selected;
-    $script->setScript("Notebook.create('$id');");
-    $self->_appendAfter($script);
+    $self->{__tabs}[0]->setSelected(1) if !$selected && @{$self->{__tabs}};
+    $self->_appendInitScript("IWL.Notebook.create('$id');");
 }
 
 sub _registerEvent {
