@@ -13,10 +13,9 @@ use IWL::Page::Link;
 use IWL::Page::Meta;
 use IWL::Page::Title;
 use IWL::Script;
-use IWL::Config qw(%IWLConfig);
 use IWL::Comment;
-
-use JSON;
+use IWL::Config qw(%IWLConfig);
+use IWL::JSON qw(toJSON);
 
 use constant DOCTYPES => {
     html401 => <<DECL,
@@ -281,10 +280,11 @@ sub __init {
     my $ie          = IWL::Page::Link->newLinkToCSS($IWLConfig{SKIN_DIR} . '/ie.css');
     my $ie6         = IWL::Page::Link->newLinkToCSS($IWLConfig{SKIN_DIR} . '/ie6.css');
     my $conditional = IWL::Comment->new;
-    $self->requiredJs('dist/prototype.js', 'dist/scriptaculous.js', 'base.js');
+    $self->requiredJs('base.js');
 
     my $script = IWL::Script->new;
-    $script->appendScript("window.IWLConfig = " . objToJson(\%IWLConfig) . ";");
+    $script->appendScript("if (!window.IWL) var IWL = {};" .
+                          "IWL.Config = " . toJSON(\%IWLConfig) . ";");
     $head->appendChild($script);
 
     $head->appendChild($skin);
