@@ -7,10 +7,14 @@ use strict;
 
 use IWL::Button;
 use IWL::File;
+use IWL::IFrame;
 use IWL::String qw(randomize);
+use IWL::JSON qw(toJSON);
 
 use base 'IWL::Form';
 
+use Locale::TextDomain qw(org.bloka.iwl);
+
 =head1 NAME
 
 IWL::Upload - a file upload widget
@@ -25,7 +29,7 @@ The Upload widget is a form widget, with it's own file selector.
 
 =head1 CONSTRUCTOR
 
-IWL::File->new ([B<%ARGS>])
+IWL::Upload->new ([B<%ARGS>])
 
 Where B<%ARGS> is an optional hash parameter with with key-values.
 
@@ -35,6 +39,20 @@ Where B<%ARGS> is an optional hash parameter with with key-values.
 
 The form action
 
+=item B<showTooltip>
+
+True, if an information tooltip should be shown
+
+=back
+
+=head1 SIGNALS
+
+=over 4
+
+=item B<upload>
+
+Fires when a file has been uploaded
+
 =back
 
 =cut
@@ -106,25 +124,12 @@ sub getLabel {
     return shift->{__button}->getLabel;
 }
 
-=item B<setUploadCallback> (B<CALLBACK>)
-
-Sets the function to be executed when a file has been uploaded.
-
-Parameters: B<CALLBACK> - the javascript callback, it will receive the json as it's parameter
-
-=cut
-
-sub setUploadCallback {
-    my ($self, $callback) = @_;
-
-    $self->{__uploadCallback} = $callback;
-    return $self;
-}
-
-=item B<printMessage> (B<MESSAGE>)
+=item B<printMessage> (B<MESSAGE>, B<DATA>)
 
 Prints a message, so that it can be displayed in the upload tooltip
 
+Parameters: B<MESSAGE> - the message to be displayed when a file is uploaded. B<DATA> - the data to be passed to a listener for the I<upload> signal.
+
 This method is a class method!  You do not need to instantiate an object
 in order to call it.
 
@@ -133,10 +138,11 @@ in order to call it.
 sub printMessage {
     my $message = shift;
     $message = shift if ref $message;
+    my $data = shift || {};
 
     my $json = IWL::Text->new;
     my $page = IWL::Page->new(simple => 1);
-    $json->setContent("{message:'$message'}");
+    $json->setContent(toJSON({message => $message, data => $data}));
     $page->appendChild($json);
     $page->print;
 }
@@ -157,17 +163,30 @@ sub setId {
     $self->setTarget($id . '_frame');
 }
 
+sub setName {
+    my ($self, $name) = @_;
+
+    $self->{__file}->setName($name);
+    return $self;
+}
+
+sub getName {
+    return shift->{__file}->getName;
+}
+
 # Protected
 #
 sub _realize {
-    my $self = shift;
-    my $id   = $self->getId;
+    my $self   = shift;
+    my $id     = $self->getId;
 
     $self->SUPER::_realize;
 
+    $self->{__file}->setStyle(visibility => 'hidden');
     my $file = $self->{__file}->getJSON;
-    my $arg = $self->{__uploadCallback} || 0;
-    $self->{__init}->setScript("Upload.create('$id', $file, {uploadCallback: window[$arg]})");
+    my $options = toJSON($self->{_options});
+    my $uploading = __"Uploading ...";
+    $self->_appendInitScript("IWL.Upload.create('$id', $file, $options, {uploading: '$uploading'})");
 }
 
 sub _setupDefaultClass {
@@ -183,10 +202,9 @@ sub _setupDefaultClass {
 #
 sub __init {
     my ($self, %args) = @_;
-    my $frame  = IWL::Widget->new;
     my $file   = IWL::File->new;
+    my $frame  = IWL::IFrame->new;
     my $button = IWL::Button->new(size => 'medium');
-    my $init   = IWL::Script->new;
 
     $self->{_defaultClass} = 'upload';
     $args{id} ||= randomize($self->{_defaultClass});
@@ -194,17 +212,18 @@ sub __init {
     $self->{__file}   = $file;
     $self->{__frame}  = $frame;
     $self->{__button} = $button;
-    $self->{__init}   = $init;
-    $self->appendChild($button);
+    $self->{_options} = {};
+    $self->{_options}{showTooltip} = $args{showTooltip} if defined $args{showTooltip};
     $self->appendChild($frame);
-    $self->appendChild($init);
+    $self->appendChild($button);
     $self->setId($args{id});
-    delete @args{qw(id)};
+    $self->setAction($args{action});
+    delete @args{qw(id action showTooltip)};
 
-    $button->setLabel('Browse ...');
-    $frame->{_tag} = 'iframe';
+    $button->setLabel(__('Browse ...'));
     $file->_constructorArguments(%args);
     $self->requiredJs('base.js', 'upload.js', 'tooltip.js');
+    $self->{_customSignals} = { upload => [] };
 
     return $self;
 }
