@@ -1,21 +1,18 @@
 // vim: set autoindent shiftwidth=4 tabstop=8:
 /**
- * @class Tooltip is a class for creating information tooltips
- * @extends Widget
+ * @class IWL.Tooltip is a class for creating information tooltips
+ * @extends IWL.Widget
  * */
-var Tooltip = {};
-Object.extend(Object.extend(Tooltip, Widget), (function() {
+IWL.Tooltip = Object.extend(Object.extend({}, IWL.Widget), (function() {
     function build(id) {
         var container;
-        if (container = $(id)) {
-            if (container.setContent)
-                this.current = container;
-            else
-                throw new Error('An element with id "' + id + '" already exists!');
-            return;
+        if ((container = $(id)) && !container.hasClassName('tooltip')) {
+            throw new Error('An element with id "' + id + '" already exists!');
+            return false;
         }
         
-        container = new Element('div', {className: 'tooltip', id: id});
+        if (!container)
+            container = new Element('div', {className: 'tooltip', id: id});
         var content = new Element('div', {className: 'tooltip_content'});
         var bubble1 = new Element('div', {className: 'tooltip_bubble tooltip_bubble_1'});
         var bubble2 = new Element('div', {className: 'tooltip_bubble tooltip_bubble_2'});
@@ -26,15 +23,7 @@ Object.extend(Object.extend(Tooltip, Widget), (function() {
         container.appendChild(bubble1);
         container.appendChild(content);
         container.style.display = 'none';
-
-        var script = $(id + '_script');
-        pivot = $(this.options.pivot);
-        if (script)
-            script.parentNode.appendChild(container);
-        else if (pivot)
-            pivot.parentNode.appendChild(container);
-        else
-            document.body.appendChild(container);
+        container.style.visibility = this.options.visibility || '';
 
         if (this.options.followMouse) {
             container.setStyle({marginTop: '5px'});
@@ -45,7 +34,19 @@ Object.extend(Object.extend(Tooltip, Widget), (function() {
         this.content = content;
         this.bubbles = new Array(bubble1, bubble2, bubble3);
 
-        return this;
+        return true;
+    }
+
+    function append() {
+        parentNode = this.options.parent == 'document.body' ? document.body : $(this.options.parent);
+        if (parentNode || !this.parentNode)
+            (parentNode || document.body).appendChild(this);
+        if (this.options.content)
+            this.setContent(this.options.content);
+        if (this.options.bind)
+            this.bindToWidget.apply(this, this.options.bind);
+        if (this.options.bindHide)
+            this.bindHideToWidget.apply(this, this.options.bindHide);
     }
 
     function draw(x, y) {
@@ -67,18 +68,29 @@ Object.extend(Object.extend(Tooltip, Widget), (function() {
             return this;
         }
 
-        var vdims = document.viewport.getDimensions();
-        var tdims = this.getDimensions();
+        var viewport_dims = document.viewport.getDimensions();
+        var scroll_offset = document.viewport.getScrollOffsets();
+        var tooltip_dims = this.getDimensions();
+        var element_top = elementRealPosition.call(this)[1];
         var compensation = this.options.followMouse ? 2.5 : 0;
         var margins = 5;
         var left = x;
         var top = y;
-        if (x < margins) left = margins;
-        if (x + tdims.width > vdims.width - margins)
-            left = vdims.width - tdims.width - margins;
-        if (y < margins) top = margins;
-        if (y + tdims.height > vdims.height - margins)
-            top = y - tdims.height - margins * 2;
+        if (Prototype.Browser.Gecko) {
+            var max_dims = document.viewport.getMaxDimensions();
+            var scroll_size = document.viewport.getScrollbarSize();
+            viewport_dims.width -= max_dims.height > viewport_dims.height ? scroll_size : 0;
+            viewport_dims.height -= max_dims.width > viewport_dims.width ? scroll_size : 0;
+        }
+
+        if (x < margins + scroll_offset.left) left = margins + scroll_offset.left;
+        if (x + tooltip_dims.width > viewport_dims.width + scroll_offset.left - margins)
+            left = viewport_dims.width + scroll_offset.left - margins - tooltip_dims.width;
+        if (y < margins + scroll_offset.top) top = margins + scroll_offset.top;
+        if (y + tooltip_dims.height > viewport_dims.height + scroll_offset.top - margins)
+            top = element_top
+                ? element_top - tooltip_dims.height
+                : viewport_dims.height + scroll_offset.top - margins - tooltip_dims.height;
 
         /* Vertical offset */
         if (top < y) {
@@ -106,13 +118,13 @@ Object.extend(Object.extend(Tooltip, Widget), (function() {
         /* Horizontal offset */
         var const_offset = bubbles[2].left + bubbles[2].width + compensation;
         var offset_x = x - left - const_offset;
-        if (offset_x > tdims.width) offset_x = tdims.width;
-        var offset_ratio = offset_x / tdims.width;
+        if (offset_x > tooltip_dims.width) offset_x = tooltip_dims.width;
+        var offset_ratio = offset_x / tooltip_dims.width;
         if (offset_ratio < 0) offset_ratio = 0;
 
-        var bubble0_x = (tdims.width - 2 * bubbles[0].left - bubbles[0].width + const_offset) * offset_ratio + bubbles[0].left;
-        var bubble1_x = (tdims.width - 2 * bubbles[1].left - bubbles[1].width + const_offset) * offset_ratio + bubbles[1].left;
-        var bubble2_x = (tdims.width - 2 * bubbles[2].left - bubbles[2].width + const_offset) * offset_ratio + bubbles[2].left;
+        var bubble0_x = (tooltip_dims.width - 2 * bubbles[0].left - bubbles[0].width + const_offset) * offset_ratio + bubbles[0].left;
+        var bubble1_x = (tooltip_dims.width - 2 * bubbles[1].left - bubbles[1].width + const_offset) * offset_ratio + bubbles[1].left;
+        var bubble2_x = (tooltip_dims.width - 2 * bubbles[2].left - bubbles[2].width + const_offset) * offset_ratio + bubbles[2].left;
 
         this.bubbles[0].style.left = bubble0_x + 'px';
         this.bubbles[1].style.left = bubble1_x + 'px';
@@ -128,24 +140,79 @@ Object.extend(Object.extend(Tooltip, Widget), (function() {
     function move(e) {
         var x = Event.pointerX(e);
         var y = Event.pointerY(e);
+        if (!this.__positioned) return;
 
         return draw.call(this, x, y);
     }
 
+    function placeAtElement() {
+        this.__positioned = false;
+        if (!this.element) return false;
+
+        var viewport_scroll = document.viewport.getScrollOffsets();
+        var viewport_dims = document.viewport.getDimensions();
+        pos = elementRealPosition.call(this);
+        pos[1] += this.element.getHeight();
+
+        if (this.options.centerOnElement)
+            pos[0] += this.element.getWidth()/2;
+
+        if (pos[0] > viewport_dims.width + viewport_scroll.left ||
+            pos[1] > viewport_dims.height + viewport_scroll.height)
+            return false;
+
+        this.__positioned = true;
+        draw.call(this, pos[0], pos[1]);
+        return true;
+    }
+
+    function elementRealPosition() {
+        if (!this.element) return [0, 0];
+        var pos = this.element.cumulativeOffset();
+        var scroll = this.element.cumulativeScrollOffset();
+        var viewport_scroll = document.viewport.getScrollOffsets();
+        if (scroll[0] || (Prototype.Browser.Opera && scroll[0] != pos[0]))
+            scroll[0] -= viewport_scroll.left;
+        if (scroll[1] || (Prototype.Browser.Opera && scroll[1] != pos[1]))
+            scroll[1] -= viewport_scroll.top;
+
+        if (Prototype.Browser.Opera) {
+            if (scroll[0] != pos[0])
+                pos[0] -= scroll[0];
+            if (scroll[1] != pos[1])
+                pos[1] -= scroll[1];
+        } else {
+            pos[0] -= scroll[0];
+            pos[1] -= scroll[1];
+        }
+
+        return pos;
+    }
+
+    function bindEvent(event, toggle) {
+        if (!this.visible() || this.__fade) {
+            this.showTooltip();
+            Event.stop(event);
+        } else if (toggle && (this.visible() || this.__appear)) {
+            this.hideTooltip();
+            Event.stop(event);
+        }
+    }
+
     return {
         /**
          * Shows the tooltip
          * @returns The object
          * */
         showTooltip: function() {
-            this.placeAtElement();
+            if (!placeAtElement.call(this)) return;
             if (this.__fade) {
                 this.__fade.cancel();
                 this.__fade = undefined;
             }
-            this.__appear = Effect.Appear(this, {
+            this.__appear = new Effect.Appear(this, {
                     duration: 0.25,
-                    afterFinish: function() { this.__appear = undefined }.bind(this)
+                    afterFinish: function() { this.__appear = undefined; this.emitSignal('iwl:show') }.bind(this)
             });
             return this;
         },
@@ -158,9 +225,9 @@ Object.extend(Object.extend(Tooltip, Widget), (function() {
                 this.__appear.cancel();
                 this.__appear = undefined;
             }
-            this._fade = Effect.Fade(this, {
+            this._fade = new Effect.Fade(this, {
                     duration: 0.5,
-                    afterFinish: function() { this.__fade = undefined }.bind(this)
+                    afterFinish: function() { this.__fade = undefined; this.emitSignal('iwl:hide') }.bind(this)
             });
             return this;
         },
@@ -174,39 +241,15 @@ Object.extend(Object.extend(Tooltip, Widget), (function() {
             if (typeof elements == 'string') {
                 this.content.update(unescape(elements));
             } else if (typeof elements == 'object') {
-                if ($A(elements).length && !(elements.nodeType == 3)) {
-                    $A(elements).each(function($_) {this.content.appendChild($_)}.bind(this));
-                } else {
-                    this.content.appendChild(elements);
-                }
+                if (Object.isElement(elements))
+                    this.content.appendChild(elements)
+                else
+                    $A(elements).each(function($_) {
+                        if (Object.isElement($_)) this.content.appendChild($_)
+                    }.bind(this));
             }
             return this;
         },
-        /**
-         * Places the tooltip at the bound element
-         * @returns The object
-         * */
-        placeAtElement: function() {
-            if (!this.element) return;
-            var pos = this.element.cumulativeOffset();
-            var scroll = this.element.cumulativeScrollOffset();
-            if (Prototype.Browser.Opera) {
-                if (scroll[0] != pos[0])
-                    pos[0] -= scroll[0];
-                if (scroll[1] != pos[1])
-                    pos[1] -= scroll[1];
-            } else {
-                pos[0] -= scroll[0];
-                pos[1] -= scroll[1];
-            }
-            pos[1] += this.element.getDimensions().height;
-
-            if (this.options.centerOnElement)
-                pos[0] += this.element.getDimensions().width/2;
-            draw.call(this, pos[0], pos[1]);
-
-            return this;
-        },
         /**
          * Removes the tooltip
          * @returns The object
@@ -227,19 +270,7 @@ Object.extend(Object.extend(Tooltip, Widget), (function() {
         bindToWidget: function(element, signal, toggle) {
             this.element = $(element);
             if (!this.element) return;
-            this.element.signalConnect(signal, function (event) {
-                if (!this.visible() || this.__fade) {
-                    this.showTooltip();
-                    Event.extend(event);
-                    if (event.stop)
-                        event.stop();
-                } else if (toggle && (this.visible() || this.__appear)) {
-                    this.hideTooltip();
-                    Event.extend(event);
-                    if ('stop' in event)
-                        event.stop();
-                }
-            }.bind(this));
+            this.element.signalConnect(signal, bindEvent.bindAsEventListener(this, toggle));
             if (!this.options.hidden)
                 this.showTooltip();
             return this;
@@ -255,9 +286,7 @@ Object.extend(Object.extend(Tooltip, Widget), (function() {
             element.signalConnect(signal, function (event) {
                 if (this.visible() || this.__appear) {
                     this.hideTooltip();
-                    Event.extend(event);
-                    if ('stop' in event)
-                        event.stop();
+                    Event.stop(event);
                 }
             }.bind(this));
             return this;
@@ -268,17 +297,28 @@ Object.extend(Object.extend(Tooltip, Widget), (function() {
                 width:      'auto',
                 centerOnElement: true,
                 hidden: false,
-                pivot: false,
-                followMouse: false
+                parent: false,
+                followMouse: false,
+                content: false,
+                bind: false,
+                bindHide: false
             }, arguments[1] || {})
-            if (!id) id = 'tooltip' + Math.random();
-            build.call(this, id);
-            return true;
+            if (!id) id = 'tooltip_' + Math.random();
+            return build.call(this, id);
         },
         _init: function() {
             if (parseInt(this.options.width))
                 this.options.width = parseInt(this.options.width) + 'px';
+
+            if (document.loaded)
+                append.call(this);
+            else
+                document.observe('dom:loaded', append.bind(this));
+
             draw.call(this);
         }
     }
 })());
+
+/* Deprecated */
+var Tooltip = IWL.Tooltip;
